'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.parse = parse;

var _htmlparser = require('htmlparser2');

var _htmlparser2 = _interopRequireDefault(_htmlparser);

var _FcCallTag = require('./FcCallTag');

var _FcContentTag = require('./FcContentTag');

var _FcEachTag = require('./FcEachTag');

var _FcIfTag = require('./FcIfTag');

var _FcElseTag = require('./FcElseTag');

var _FcElseIfTag = require('./FcElseIfTag');

var _Statements = require('./Statements');

var _Factory = require('./Factory');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @typedef {object} BUILT_IN_TAGS
 * @desc The built in tags provided by funclate.
 * @property {FcEachTag} fc-each a forEach loop implementation
 * @property {FcIfTag} fc-if a if clause implementation
 * @property {FcElseTag} fc-else a else clause implementation
 * @property {FcElseIfTag} fc-if-else a if-else clause implementation
 * @property {FcElseIfTag} fc-content A funclate's tag to specify a content node.
 */

var tags = {
    'fc-each': new _FcEachTag.FcEachTag(),
    'fc-if': new _FcIfTag.FcIfTag(),
    'fc-else': new _FcElseTag.FcElseTag(),
    'fc-else-if': new _FcElseIfTag.FcElseIfTag(),
    'fc-content': new _FcContentTag.FcContentTag(),
    'fc-call': new _FcCallTag.FcCallTag()
};

/**
 * @typedef {object} ParserOptions
 * @desc The override-able options of funclate.
 * @property {string} output the output format <code>HTML</code> or <code>function</code>
 * @property {boolean} pretty to add <code>\n</code> after between Javascript statements
 * @property {RegExp} interpolation RegExp to inject interpolated values.
 * @property {string} propNamePrefix the prefix value of attribute name to identified properties
 * @property {string} elVarName the variable name for the element
 * @property {string} ctxVarName the variable name for the context
 * @property {Array<string>} selfClosingElements The list of self closing elements. (http://www.w3.org/TR/html5/syntax.html#void-elements)
 * @property {BUILT_IN_TAGS} tags The built in and custom tags.
 */

/**
 * @type ParserOptions
 */
var DEFAULT_OPTIONS = {
    output: 'function',
    pretty: false,
    interpolation: /\{\{([\s\S]+?)}}/gm,
    propNamePrefix: '#',
    elVarName: 'el',
    ctxVarName: 'ctx',
    selfClosingElements: ['area', 'base', 'br', 'col', 'embed', 'hr', 'img', 'input', 'keygen', 'link', 'meta', 'param', 'source', 'track', 'wbr'],
    tags: tags
};

/**
 * Parse an HTML template and return the factory of the underlying render function.
 * @param {!string} html the HTML content to parse
 * @param {ParserOptions} [options] the options
 * @return {function|string} the factory function as Function or string.
 */
function parse(html, options) {
    options = (0, _utils.assign)({}, DEFAULT_OPTIONS, options);

    var factory = new _Factory.Factory(options);
    var p = new _htmlparser2.default.Parser({
        onopentag: function onopentag(name, attrs, selfClosing) {
            if (options.tags[name]) {
                options.tags[name].startTag(factory, name, attrs, selfClosing);
            } else {
                var fcAttrs = _Statements.Statements.get(options);
                var fcProps = _Statements.Statements.get(options);
                var fcOpts = _Statements.Statements.get(options);
                Object.keys(attrs).forEach(function (attName) {
                    if (attName === 'fc-content') {
                        fcOpts.append('\'content\', true');
                    }
                    var index = attName.indexOf(options.propNamePrefix);
                    var name = attName;
                    var destination = fcAttrs;
                    if (index > -1) {
                        name = (0, _utils.toCamelCase)(attName.substring(index + 1));
                        destination = fcProps;
                    }
                    destination.append('\'' + name + '\', ' + (0, _utils.interpolate)(attrs[attName], options));
                });
                if (options.selfClosingElements.indexOf(name) > -1) {
                    factory.appendVoidElement(name, '[' + fcAttrs.join(',') + ']', '[' + fcProps.join(',') + ']', '[' + fcOpts.join(',') + ']');
                } else {
                    factory.appendOpenElement(name, '[' + fcAttrs.join(',') + ']', '[' + fcProps.join(',') + ']', '[' + fcOpts.join(',') + ']');
                }
            }
        },
        onclosetag: function onclosetag(name) {
            if (options.tags[name]) {
                options.tags[name].endTag(factory, name);
            } else if (options.selfClosingElements.indexOf(name) < 0) {
                factory.appendCloseElement();
            }
        },
        ontext: function ontext(text) {
            factory.appendText((0, _utils.interpolate)(text, options));
        },
        oncomment: function oncomment(text) {
            factory.appendComment((0, _utils.interpolate)(text, options));
        }
    }, {
        xmlMode: false,
        decodeEntities: false,
        lowerCaseTags: true,
        lowerCaseAttributeNames: true,
        recognizeSelfClosing: true,
        recognizeCDATA: true
    });

    p.parseComplete(html);
    return factory.toFunction();
}