'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.createThenUpdate = createThenUpdate;
exports.updateElement = updateElement;
exports.openElement = openElement;
exports.closeElement = closeElement;
exports.voidElement = voidElement;
exports.content = content;
exports.text = text;
exports.comment = comment;

var _utils = require('./utils');

/**
 * The current root element, i.e. where the starting point of the updateElement function.
 * @type {Element}
 */
var rootElement = null;

/**
 * The current document owning the current root element
 * @type {Document}
 */
var rootDocument = null;

/**
 * The current parent element of the processed instruction.
 * @type {Element}
 */
var parentElement = null;

/**
 * @typedef {object} Context
 * @property {Element} root the root element
 * @property {Element} parent the parent element
 * @property {Document} document the document owning the root element
 * @private
 */

/**
 * @typedef {object} ElementOptions
 * @property {boolean} content if true the element will be the entry of the light DOM structure
 * @property {string} key the element's key
 * @property {boolean} skipChildren if true the children's element won't be parsed
 * @property {Node} found will be used as current node
 * @private
 */

/**
 * Build a context object.
 * @return {Context} the context
 */
function getCtx() {
    return { root: rootElement, parent: parentElement, document: rootDocument };
}

/**
 * Restore a context object.
 * @param {!Context} ctx the context to restore
 */
function restoreCtx(ctx) {
    rootElement = ctx.root;
    parentElement = ctx.parent;
    rootDocument = ctx.document;
}

/**
 * Remove the un-visited nodes.
 */
function cleanRemainingNodes() {
    if (rootElement.__content__ !== parentElement) {
        while (parentElement.childNodes.length > lastIndex()) {
            parentElement.removeChild(parentElement.lastChild);
        }
    }
}

/**
 * Get and increment the fcIndex property of the current parent.
 * @return {number} the index
 */
function nextIndex() {
    if (!parentElement.fcIndex) {
        parentElement.fcIndex = 0;
    }
    var index = parentElement.fcIndex;
    parentElement.fcIndex = parentElement.fcIndex + 1;
    return index;
}

/**
 * Get fcIndex property of the current parent.
 * @return {number} the index
 */
function lastIndex() {
    return parentElement.fcIndex || 0;
}

/**
 * Clear the fcIndex property of the current parent.
 */
function clearIndex() {
    delete parentElement.fcIndex;
}

/**
 * Handle a Node.
 * @param {!string} value the node's value
 * @param {!number} nodeType the node's type
 * @param {!function(value: string): Node} factory the factory creating the node when necessary
 * @return {Node} the handled node
 */
function handleNode(value, nodeType, factory) {
    var index = nextIndex();
    var current = parentElement.childNodes.item(index);

    if (current) {
        if (current.nodeType !== nodeType) {
            parentElement.insertBefore(factory(value), current);
        } else {
            current.nodeValue = value;
        }
    } else {
        parentElement.appendChild(factory(value));
    }

    return current;
}

/**
 * Create an element
 * @param {!string} name the name
 * @param {Object.<string, string|number|boolean>} attrs the attributes
 * @return {Element} the element
 */
function createElement(name, attrs) {
    var index = attrs ? attrs.indexOf('is') : -1;
    return index > -1 ? rootDocument.createElement(name, attrs[index + 1]) : rootDocument.createElement(name);
}

/**
 * Handle a {Element}.
 * @param {!string} name the name of the element
 * @param {Array} [attrs] the attributes of the element
 * @param {Array}  [props] the properties of the element
 * @param {ElementOptions} [opts] the options driving the creation of the element
 * @return {Element} the handled element
 */
function handleElement(name, attrs, props, opts) {
    var index = nextIndex();
    var current = parentElement.childNodes.item(index);

    var found = opts.found ? opts.found : null;
    if (!found && opts.key) {
        found = (0, _utils.findNodeFromKey)(parentElement, lastIndex(), opts.key);
    }

    if (found) {
        if (current !== found) {
            current = parentElement.insertBefore(found, current);
        }
    } else if (current) {
        if (name.toLowerCase() !== (current.tagName || '').toLowerCase()) {
            current = parentElement.insertBefore(createElement(name, attrs), current);
        }
    } else {
        current = parentElement.appendChild(createElement(name, attrs));
    }

    (0, _utils.updateAttributes)(current, attrs);
    (0, _utils.updateProperties)(current, props);

    if (opts.key) {
        current.dataset.fcKey = opts.key;
    }

    if (opts.content) {
        rootElement.__content__ = current;
    }

    if (!opts.skipChildren) {
        parentElement = current;
    }

    return current;
}

/**
 * List all methods.
 * This object should be used as argument of factories of render functions.
 * @type {{closeElement: closeElement, openElement: openElement, comment: comment, content: content, text: text, voidElement: voidElement}}
 */
var METHODS = { closeElement: closeElement, openElement: openElement, comment: comment, content: content, text: text, voidElement: voidElement };

/**
 * Update the sub tree of the given root according to a given factory of the render function.
 * @param {!function(fc: object)} factory the render function
 * @param {!HTMLElement} root the root element
 * @return {function(el: HTMLElement)}  the render function
 * @param {object} context an optional context
 */
function createThenUpdate(factory, root) {
    var context = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    var render = factory(METHODS);
    updateElement(factory(METHODS), root, context);
    return render;
}

/**
 * Update the sub tree of the given root according to the given render function.
 * @param {!function(el: HTMLElement)} render the render function
 * @param {!HTMLElement} root the root element
 * @param {object} context an optional context
 */
function updateElement(render, root) {
    var context = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    var ctx = getCtx();

    rootElement = root;
    rootDocument = root.ownerDocument;
    parentElement = root;

    var lightFrag = null;
    if (!root.__visited__) {
        lightFrag = rootDocument.createDocumentFragment();
        while (root.childNodes.length > 0) {
            lightFrag.appendChild(root.removeChild(root.firstChild));
        }
    }

    render(root, context);

    cleanRemainingNodes();
    clearIndex();

    root.__visited__ = true;
    if (lightFrag && root.__content__) {
        root.__content__.appendChild(lightFrag);
    }

    restoreCtx(ctx);
}

/**
 * Open an element.
 *
 * @example <caption>set attributes</caption>
 * openElement('p', ['id', 'my-p' ,'class', classValue]);
 * closeElement();
 *
 * @example <caption>set properties</caption>
 * const selectValue = 'foo';
 * openElement('select', null, ['value', selectValue])
 * closeElement();
 *
 * @example <caption>set the element has a light DOM content</caption>
 * openElement('div', null, null, ['content', true])
 * closeElement();
 *
 * @example <caption>set the element's key</caption>
 * openElement('li', null, null, ['key', 'my-li'])
 * closeElement();
 *
 * @param {!string} name the name of the element
 * @param {Array.<undefined|null|string|number|boolean>} [attrs] the attributes of the element
 * @param {Array} [props] the properties of the element
 * @param {Array} [opts] the options driving the creation of the element, c.f. {@link ElementOptions}
 * @return {Element} the element
 */
function openElement(name, attrs, props, opts) {
    return handleElement(name, attrs, props, (0, _utils.fromArrayToObject)(opts));
}

/**
 * Close the current element.
 */
function closeElement() {
    cleanRemainingNodes();
    clearIndex();
    parentElement = parentElement.parentElement;
}

/**
 * Open and close a void element.
 * @param {!string} name the name of the element
 * @param {Array.<undefined|null|string|number|boolean>} [attrs] the attributes of the element
 * @param {Array} [props] the properties of the element
 * @param {Array} [opts] the options driving the creation of the element, c.f. {@link ElementOptions}
 * @return {Element} the element
 */
function voidElement(name, attrs, props, opts) {
    return handleElement(name, attrs, props, (0, _utils.fromArrayToObject)(['skipChildren', true].concat(opts)));
}

/**
 * To add a <code><fc-content></fc-content></code> node.
 * This node will be used to locate the entry of the light DOM structure.
 */
function content() {
    rootElement.__content__ = handleElement('fc-content', null, null, {
        skipChildren: true,
        found: rootElement.__content__
    });
}

/**
 * Create a text node.
 * @param {string} [value] the node value
 * @returns {Text}
 */
function textNodeFactory(value) {
    return rootDocument.createTextNode((0, _utils.sanitize)(value));
}

/**
 * Create a text node.
 * @param {string} [value] the node value
 * @returns {Comment}
 */
function commentFactory(value) {
    return rootDocument.createComment((0, _utils.sanitize)(value));
}

/**
 * To write a text node.
 * @param {string} [text] the text
 * @return {Node} the text node
 */
function text(text) {
    return handleNode(text, rootElement.TEXT_NODE, textNodeFactory);
}

/**
 * To write a comment node.
 * @param {string} [text] the comment
 * @return {Node} the comment node
 */
function comment(text) {
    return handleNode(text, rootElement.COMMENT_NODE, commentFactory);
}