'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.toArray = toArray;
exports.sanitize = sanitize;
exports.fromArrayToObject = fromArrayToObject;
exports.findNodeFromKey = findNodeFromKey;
exports.updateAttributes = updateAttributes;
exports.updateProperties = updateProperties;
/**
 * Convert a value to an array.
 * @param {*} value the value to convert
 * @returns {Array} the array
 * @private
 */
function toArray(value) {
    return value ? Array.prototype.slice.call(value) : [];
}

/**
 * Convert undefined and null value to empty string.
 * @param {string} value the value to sanitize
 * @returns {string} the sanitized value
 * @private
 */
function sanitize(value) {
    return value === undefined || value === null ? '' : value;
}

/**
 * Transform an array to an object.
 * The first value is the key, the second its value and so one.
 * @param {Array} array the array
 * @returns {object} the object
 * @private
 */
function fromArrayToObject(array) {
    var object = {};
    if (!array) {
        return object;
    }
    var max = array.length;
    for (var i = 0; i < max; i = i + 2) {
        object[array[i]] = array[i + 1];
    }
    return object;
}

/**
 * According to the given parent, try to find a node matching with the given key.
 * @param {!Element} parent the parent element
 * @param {!number} from the index of the starting node
 * @param {!string} key the key to find
 * @returns {Node}
 * @private
 */
function findNodeFromKey(parent, from, key) {
    var end = parent.childNodes.length;
    for (var i = from; i < end; i++) {
        var child = parent.childNodes.item(i);
        if (child.dataset && child.dataset.fcKey === key) {
            return child;
        }
    }
}

/**
 * Update the given element's attributes according to the new one.
 * @param {!Element} element the element
 * @param {Array.<undefined|null|string|number|boolean>} [attributes] the new attributes
 * @private
 */
function updateAttributes(element, attributes) {
    attributes = attributes ? attributes : [];
    var updatedAttributes = {};
    var max = attributes.length;
    for (var i = 0; i < max; i = i + 2) {
        var name = attributes[i];
        var value = attributes[i + 1];
        var type = typeof value === 'undefined' ? 'undefined' : _typeof(value);
        if (type === 'boolean') {
            if (value) {
                element.setAttribute(name, '');
                updatedAttributes[name] = true;
            }
        } else if (type === 'number') {
            element.setAttribute(name, value);
            updatedAttributes[name] = true;
        } else if (value) {
            element.setAttribute(name, sanitize(value));
            updatedAttributes[name] = true;
        }
    }
    toArray(element.attributes).filter(function (attr) {
        return !updatedAttributes[attr.name];
    }).forEach(function (attr) {
        return element.removeAttribute(attr.name);
    });
}
/**
 * Update the given element's properties according to the new one.
 * @param {!Element} element the element
 * @param {Array} [properties] the new attributes
 * @private
 */
function updateProperties(element, properties) {
    properties = properties ? properties : [];
    var updatedProperties = {};
    var max = properties.length;
    for (var i = 0; i < max; i = i + 2) {
        var name = properties[i];
        element[name] = properties[i + 1];
        updatedProperties[name] = true;
    }
    toArray(element.updatedProperties).filter(function (name) {
        return !updatedProperties[name];
    }).forEach(function (name) {
        return element[name] = undefined;
    });
    element.updatedProperties = Object.keys(updatedProperties);
}